"""
Provides systembridgebackend version information.
"""

# This file is auto-generated! Do not edit!
# Use `python -m incremental.update systembridgebackend` to change this file.

from incremental import Version

__version__ = Version("systembridgebackend", 3, 1, 5)
__all__ = ["__version__"]
